UNIT FSCHEIN;

(* ******************************************************************** *)
(* K L A S S E : TFSchein                                               *)
(* -------------------------------------------------------------------- *)
(* Version     : 3.1                                                    *)
(*                                                                      *)
(* Autor       : (c) S. Spolwig, OSZ Handel, 10997 Berlin               *)
(*                                                                      *)
(* Dateiname   : fschein.pas                                            *)
(*                                                                      *)
(* Aufgabe     : bildet einen zunaechst leeren Fahrschein ab, der mit   *)
(*               erforderlichen Informationen bedruckt wird             *)
(*                                                                      *)
(* Compiler    : Turbo Pascal 7.0 / MSDOS                               *)
(* Aenderung   : V. 1.1 -  23-JAN-98  HatPreis entfernt, TarifInit      *)
(* Version     :    3.1 -  01-FEB-98  BILDSCHIRM, GRAFIK eingebunden    *)
(* ******************************************************************** *)

INTERFACE
(* ========================== Export ================================== *)

USES FENSTER,
     GRAFIK;

type
  CFahrschein = object (CFenster)
                  Papier      : CFlaeche;
                  Tarif,
                  Preis       : word;
                  Bezeichnung : string;

                  constructor Init (sp,zei,lae,br, einRahmen : integer ;
                                    einTitel : string);

                  procedure   TarifInit(einTarif, einPreis : word;
                                        eineBezeichnung : string); virtual;

                  procedure   Zeigen;   virtual;
                  procedure   Loeschen; virtual;
                end;

(*  ----------------- B e s c h r e i b u n g -------------------------

Oberklasse   : CFenster
Bezugsklassen: CTarifliste, CBildschirm, Grafik
Module       : -


Methoden
--------

Init

  Auftrag: Alle Attribute festlegen
  vorher : -
  nachher: Position,Groesse festlegen; Tarif, PrBez. und Preis uebnergeben


TarifInit(einTarif, einPreis: word; eineBezeichnung : string)

  Auftrag: Fahrschein mit den erfoderlichen Informationen versehen
  vorher : init.
  nachher: Tarif, Preis und Bezeichnung sind definiert


Zeigen

  Auftrag: Fahrschein auf dem Bildschirm zeigen.
  vorher : init.
  nachher: Fahrschein steht auf dem Bildschirm

----------------------------------------------------------------------- *)

IMPLEMENTATION
(* ==================================================================== *)
USES
    BILDSCHIRM;

var
   Schatten : CFlaeche;

constructor CFahrschein.Init (sp,zei,lae,br, einRahmen : integer; einTitel : string);
(* -------------------------------------------------------------------- *)
begin
  CFenster.Init (sp,zei,lae,br, einRahmen, einTitel);
  Papier.Init (XPos+1,YPos+1,Laenge,Hoehe);
  Papier.FarbeSetzen(SCHWARZ);               (* der Schatten *)

  Papier.Init (XPos,YPos,Laenge,Hoehe);
  Papier.FarbeSetzen(WEISS);
  Papier.ZeichenFarbeSetzen(GELB);
  Papier.ZeichenSetzen(chr(176));
end;


procedure CFahrschein.TarifInit (einTarif, einPreis: word; eineBezeichnung : string);
(* -------------------------------------------------------------------- *)
begin
  Tarif       := einTarif;
  Preis       := einPreis;
  Bezeichnung := eineBezeichnung;
end;


procedure CFahrschein.Zeigen;
(* -------------------------------------------------------------------- *)
begin
  Schatten.Init (XPos+1,YPos+1,Laenge,Hoehe);
  Schatten.FarbeSetzen(SCHWARZ);               (* der Schatten *)
  Schatten.Zeigen;
  Papier.Zeigen;

  Screen.Cursor (XPos+2, YPos+1); write(' FAHRSCHEIN ');
  Screen.Cursor (XPos+2, YPos+2); write(Bezeichnung);
  Screen.Cursor (XPos+9, YPos+4); write(Preis/100 :2:2);
  Screen.NormalVideo;
  Screen.Cursor (XPos+4, YPos+6); write(' B V B ');
  Screen.CursorHome;
end;

procedure CFahrschein.Loeschen;
(* -------------------------------------------------------------------- *)

begin
  CFenster.Loeschen;
  Schatten.Loeschen;
end;

BEGIN
END. (* FSCHEIN *)