UNIT MGEBER;

(* ******************************************************************** *)
(* K L A S S E : TMuenzgeber                                            *)
(* -------------------------------------------------------------------- *)
(* Version     : 1.1                                                    *)
(*                                                                      *)
(* Autor       : (c) S. Spolwig, OSZ Handel I, 10997 Berlin             *)
(*                                                                      *)
(* Dateiname   : mgeber.pas                                             *)
(*                                                                      *)
(* Aufgabe     : Stellt das Bauteil zur Annahme, Pruefung und Ausgabe   *)
(*               von Muenzen dar.                                       *)
(*               Es berechnet aus dem Rueckgeld die minimale            *)
(*               Stueckzahl an Muenzen und wirft sie aus.               *)
(*               Die Rueckgabe erfolgt nur in Muenzen                   *)
(*                                                                      *)
(* Compiler    : Turbo Pascal 7.0 / MSDOS                               *)
(* Aenderung   : V. 1.1 - 23-JAN-98   neuer Bezeichner: GeldAnnehmen    *)
(* ******************************************************************** *)

INTERFACE
(* ========================== Export ================================== *)

USES
  GEBER,
  MUENZE;

type
   TMuenzgeber = object (CGeber)
                   Muenzschlitz  : CGeber;
                   Muenze        : TMuenze;
                   Summe         : integer;
                   Geldtaste     : char;

                   constructor Init (sp,zei,lae,br, einRahmen : integer;
                                     einTitel : string);

                   procedure Zeigen;                        virtual;
                   procedure GeldAnnehmen (fspreis : word); virtual;
                   procedure Einwerfen; virtual;
                   procedure Speichern; virtual;
                   procedure Reset;     virtual;
                   procedure RueckgeldAuszahlen;           virtual;
                   procedure Auswerfen (anzahl : integer); virtual;
                   function  IstRueckgeld : integer;        virtual;
                 end;

(*  ----------------- B e s c h r e i b u n g -------------------------

Oberklasse     : CGeber
Bezugsklassen  : CBildschirm, CDisplay, CMuenze
benutzte Module: -


Methoden
--------

Init (sp,zei,lae,br, einRahmen : integer ; einTitel : string);

  Aufgabe:
  vorher : -
  nachher: -


Zeigen

  Aufgabe: Muenzgeber und Muenzschlitz auf den Bildschirm
  vorher : -
  nachher: -


GeldAnnehmen

  Aufgabe: Eingeworfene (eingetippte) Sorten einlesen,
           pruefen, Falschmuenzen abweisen, Betrag speichern
  vorher : -
  nachher: -


Speichern

  Aufgabe: Eingeworfene Muenzen speichern und aufsummieren
  vorher : Muenzgeber ist Reset gestartet
  nachher: Summe enthaelt eingeworefenen Betrag


Reset

  Aufgabe: Aufgelaufene Betraege wieder zuruecksetzen
  vorher : -
  nachher: Speicher ist 0


IstRueckgeld

  Anfrage: Aus eingeworfenen Muenzen und gewaehltem Fahrpreis das Rueckgeld
           berechnen und zurueckliefern
  vorher : -
  nachher: -


RueckgeldAuszahlen

  Aufgabe : Berechnen, wieviele Stuecke jeweils auszugeben sind und dann
            auswerfen
  vorher  : init.
  nachher : -


Auswerfen

  Aufgabe : Muenzen im Muenzgeber zeigen
  vorher  : init.
  nachher : -

----------------------------------------------------------------------- *)


IMPLEMENTATION
(* ==================================================================== *)

USES
  BILDSCHIRM,
  CRT,         (* import : readkey *)
  DISPL;


const
   EXPos = 40;  (* Muenzschlitz links, oben *)
   EYPos = 15;

var
  Groschen,
  Fuenfziger,
  Mark,
  Zweimark,
  Fuenfmark,
  FalscherFuffziger : TMuenze;

  Gesamtstueckzahl,
  Fahrscheinpreis,

  ZielXPos,
  AusgabeYPos      : integer;   (* im Muenzgeber *)


constructor TMuenzgeber.Init (sp,zei,lae,br, einRahmen : integer; einTitel : string);
(* -------------------------------------------------------------------- *)
begin
  CGeber.Init (sp,zei,lae,br, einRahmen, einTitel);

  Muenzschlitz.Init(EXPos,EYPos,3,3,3,'');
  Muenze.Init     (2,0,'', HELLGRAU);

  Groschen.Init   ( 2,  10, ' 10 ' , TUERKIS );
  Fuenfziger.Init ( 2,  50, ' 50 ' , TUERKIS );
  Mark.Init       ( 2, 100, ' 1M ', TUERKIS );
  Zweimark.Init   ( 2, 200, ' 2M ', TUERKIS );
  Fuenfmark.Init  ( 2, 500, ' 5M ', TUERKIS );
end;


procedure TMuenzgeber.Zeigen;
(* -------------------------------------------------------------------- *)
begin
  CGeber.Zeigen;
  Muenzschlitz.Zeigen;
end;


procedure TMuenzgeber.Reset;
(*-------------------------------------------------------------------- *)
begin
  Summe := 0;
end;


procedure TMuenzgeber.Einwerfen;
(* -------------------------------------------------------------------- *)

var i : integer;

begin
  repeat
    Geldtaste := readkey;
  until Geldtaste IN ['a'..'z', 'A'..'Z'];

  case (Geldtaste) of

   'f' :  Muenze := fuenfziger;
   'g' :  Muenze := groschen;
   'm' :  Muenze := mark;
   'z' :  Muenze := zweimark;
   'F' :  Muenze := fuenfmark;

   else
     begin
       FalscherFuffziger.Init (2,0, ' '+ Geldtaste+' ', ROT);
       Muenze := FalscherFuffziger;
     end;
  end; (* case *)
  Muenze.PositionSetzen(EXPos-5, EYPos + 1);
  Muenze.Zeigen;
  delay(400);

  for i := 1 to 4 do      (* und rein damit *)
  begin
    Muenze.VersetzenUm(1,0);
    Screen.VerzoegernUm(200);
  end;
  Screen.VerzoegernUm(200);
  Muenze.Loeschen;
  Muenzschlitz.Zeigen;
end;


procedure TMuenzgeber.Speichern;
(* -------------------------------------------------------------------- *)
begin
  Summe := Summe + Muenze.HatWert;
end;


procedure TMuenzgeber.GeldAnnehmen ( fspreis : word);
(* -------------------------------------------------------------------- *)
begin
  Reset;
  Fahrscheinpreis := fspreis;
  Display.Leeren;
  Display.TextZeigen('Bitte Geld einwerfen');

  repeat
    Display.ZahlZeigen(Summe/100);

    Einwerfen;

    if Muenze.IstEcht
    then Speichern
    else Auswerfen(1);

  until Summe >= Fahrscheinpreis;

  Display.Leeren;
  Display.TextZeigen('Ihre Zahlung');
  Display.ZahlZeigen(Summe/100);
end; (* GeldAnnehmen *)


function TMuenzgeber.IstRueckgeld : integer;
(* -------------------------------------------------------------------- *)
begin
  IstRueckgeld := Summe - Fahrscheinpreis;
end;


procedure TMuenzgeber.Auswerfen (anzahl : integer);
(* -------------------------------------------------------------------- *)
var
  zaehler,
  i        : integer;

begin
  for zaehler := 1 to anzahl do
  begin
    if Muenze.IstEcht
    then begin
      Muenze.PositionSetzen(ZielXPos, ausgabeYPos);
      Muenze.Zeigen;
      Screen.Beep(1);
      Screen.VerzoegernUm(500);

      if Gesamtstueckzahl <= 5
      then zielXPos := zielXPos + 7
      else
       begin
          zielXPos := zielXPos - 5;
          AusgabeYPos := YPOs + 1;
       end;
    end (* istEcht *)
    else
      begin
          ZielXPos := XPos + 15;
          AusgabeYPos := YPos + 1;
          for i := 1 to 3 do
          begin
            Muenze.PositionSetzen (ZielXPos, AusgabeYPos);
            Muenze.Zeigen;
            Screen.Beep(1);
            Screen.VerzoegernUm(700);
            Leeren;
            zielXPos := zielXPos +1;
            AusgabeYPos := ausgabeYPos+1;
          end;
        end;     (* else *)
  end;  (* for *)
end;


procedure TMuenzgeber.RueckgeldAuszahlen;
(* -------------------------------------------------------------------- *)
var
  Sorte      : (FMA, ZMA, EMA, FZG, GRO);

  Stueckzahl,
  Rest       : integer;

begin
  ZielXPos    := XPos + 2;
  AusgabeYPos := YPos + 2;
  GesamtStueckzahl := 0;
  Rest := IstRueckgeld;

  for Sorte := FMA to GRO do        (* alle Sorten pruefen *)
  begin
    Stueckzahl := 0;
    case Sorte of

      FMA :  muenze := Fuenfmark;   (* wie oft jeweils aus-*)
      ZMA :  muenze := Zweimark;    (* gegeben werden kann *)
      EMA :  muenze := Mark;
      FZG :  muenze := Fuenfziger;
      GRO :  muenze := Groschen;
      else;;
    end; (* case Sorte *)

    Stueckzahl := Rest DIV Muenze.HatWert;
    Rest := Rest - (Stueckzahl * Muenze.Hatwert);
    GesamtStueckzahl := GesamtStueckzahl + Stueckzahl;

    if Stueckzahl > 0
    then Auswerfen (Stueckzahl);
  end; (* for *)
end;

BEGIN
END. (* MGEBER *)